@@ -1,7 +1,7 @@
 /*
  * PHP-GTK - The PHP language bindings for GTK+
  *
- * Copyright (C) 2001 Andrei Zmievski <andrei@php.net>
+ * Copyright (C) 2001-2004 Andrei Zmievski <andrei@php.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: php_gtk+_types.c,v 1.14 2001/11/13 09:17:43 mfischer Exp $ */
+/* $Id: php_gtk+_types.c,v 1.25.4.2 2004/08/07 06:49:34 andrei Exp $ */
 
 #include "php_gtk.h"
 
@@ -273,14 +273,17 @@ PHP_FUNCTION(gdk_window_get_pointer)
 
 PHP_FUNCTION(gdk_window_set_cursor)
 {
-	zval *cursor;
+	zval *php_cursor = NULL;
+	GdkCursor *cursor = NULL;
 
 	NOT_STATIC_METHOD();
 
-	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "O", &cursor, gdk_cursor_ce))
+	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "|O", &php_cursor, gdk_cursor_ce))
 		return;
 
-	gdk_window_set_cursor(PHP_GDK_WINDOW_GET(this_ptr), PHP_GDK_CURSOR_GET(cursor));
+	if (php_cursor)
+		cursor = PHP_GDK_CURSOR_GET(php_cursor);
+	gdk_window_set_cursor(PHP_GDK_WINDOW_GET(this_ptr), cursor);
 	RETURN_NULL();
 }
 
@@ -366,7 +369,9 @@ PHP_FUNCTION(gdk_window_new_gc)
 		}
 	}
 
-	gc = gdk_gc_new_with_values(PHP_GDK_WINDOW_GET(this_ptr), &values, mask);
+	gc = gdk_gc_new_with_values(Z_OBJCE_P(this_ptr) == gdk_bitmap_ce ?
+								PHP_GDK_BITMAP_GET(this_ptr) :
+								PHP_GDK_WINDOW_GET(this_ptr), &values, mask);
 	*return_value = *php_gdk_gc_new(gc);
 	gdk_gc_unref(gc);
 }
@@ -484,7 +489,7 @@ PHP_FUNCTION(gdk_window_property_change)
 					 zend_hash_get_current_data(Z_ARRVAL_P(php_data), (void **)&data_temp);
 					 zend_hash_move_forward(Z_ARRVAL_P(php_data))) {
 					convert_to_long_ex(data_temp);
-					data16[i++] = Z_LVAL_PP(data_temp); 
+					data16[i++] = (guint16)Z_LVAL_PP(data_temp); 
 				}
 			}	
 			break;
@@ -566,6 +571,35 @@ PHP_FUNCTION(gdk_window_set_icon)
 	gdk_window_set_icon(PHP_GDK_WINDOW_GET(this_ptr), icon_window, pixmap, mask);
 }
 
+PHP_FUNCTION(gdk_window_copy_area)
+{
+	zval *gc, *php_src_window;
+	GdkDrawable *src_window = NULL;
+	int x, y, src_x, src_y, width, height;
+
+	NOT_STATIC_METHOD();
+
+	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "OiiViiii", &gc, gdk_gc_ce, &x, &y,
+							&php_src_window, &src_x, &src_y, &width, &height))
+		return;
+
+	if (Z_TYPE_P(php_src_window) != IS_NULL) {
+		if (php_gtk_check_class(php_src_window, gdk_window_ce))
+			src_window = (GdkDrawable *)PHP_GDK_WINDOW_GET(php_src_window);
+		else if(php_gtk_check_class(php_src_window, gdk_pixmap_ce))
+			src_window = (GdkDrawable *)PHP_GDK_PIXMAP_GET(php_src_window);
+		else if(php_gtk_check_class(php_src_window, gdk_bitmap_ce))
+			src_window = (GdkDrawable *)PHP_GDK_BITMAP_GET(php_src_window);
+		else {
+			php_error(E_WARNING, "%s() expects source drawable to be GdkWindow, GdkPixmap, or GdkBitmap", get_active_function_name(TSRMLS_C));
+			return;
+		}
+	}
+
+	gdk_window_copy_area(PHP_GDK_WINDOW_GET(this_ptr), PHP_GDK_GC_GET(gc), x, y,
+						 src_window, src_x, src_y, width, height);
+}
+
 static function_entry php_gdk_window_functions[] = {
 	{"GdkWindow",		PHP_FN(no_direct_constructor), NULL},
 	{"gdkwindow",		PHP_FN(no_direct_constructor), NULL},
@@ -578,17 +612,26 @@ static function_entry php_gdk_window_functions[] = {
 	{"property_change", PHP_FN(gdk_window_property_change), NULL},
 	{"property_delete", PHP_FN(gdk_window_property_delete), NULL},
 	{"set_icon",		PHP_FN(gdk_window_set_icon), NULL},
+	{"copy_area",		PHP_FN(gdk_window_copy_area), NULL},
+	{NULL, NULL, NULL}
+};
+
+static function_entry php_gdk_bitmap_functions[] = {
+	{"GdkBitmap",		PHP_FN(no_direct_constructor), NULL},
+	{"gdkbitmap",		PHP_FN(no_direct_constructor), NULL},
+	{"new_gc", 			PHP_FN(gdk_window_new_gc), NULL},
+	{"copy_area",		PHP_FN(gdk_window_copy_area), NULL},
 	{NULL, NULL, NULL}
 };
 
 PHP_FUNCTION(gdkpixmap)
 {
 	GdkWindow *window = NULL;
-	zval *php_window, *ret;
-	long width, height, depth;
+	zval *php_window;
+	long width, height, depth = -1;
 	GdkPixmap *wrapped_obj = NULL;
 
-	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "Niii", &php_window, gdk_window_ce, &width, &height, &depth)) {
+	if (!php_gtk_parse_args(ZEND_NUM_ARGS(), "Nii|i", &php_window, gdk_window_ce, &width, &height, &depth)) {
 		php_gtk_invalidate(this_ptr);
 		return;
 	}
@@ -613,6 +656,7 @@ static function_entry php_gdk_pixmap_functions[] = {
 	{"property_get", 	PHP_FN(gdk_window_property_get), NULL},
 	{"property_change", PHP_FN(gdk_window_property_change), NULL},
 	{"property_delete", PHP_FN(gdk_window_property_delete), NULL},
+	{"copy_area",		PHP_FN(gdk_window_copy_area), NULL},
 	{NULL, NULL, NULL}
 };
 
@@ -642,7 +686,7 @@ zval *php_gdk_window_new(GdkWindow *window)
 	return result;
 }
 
-zval *php_gdk_pixmap_new(GdkPixmap *pixmap)
+PHP_GTK_API zval *php_gdk_pixmap_new(GdkPixmap *pixmap)
 {
 	zval *result;
 	zval **wrapper_ptr;
@@ -668,7 +712,7 @@ zval *php_gdk_pixmap_new(GdkPixmap *pixmap)
 	return result;
 }
 
-zval *php_gdk_bitmap_new(GdkWindow *bitmap)
+PHP_GTK_API zval *php_gdk_bitmap_new(GdkWindow *bitmap)
 {
 	zval *result;
 	zval **wrapper_ptr;
@@ -696,55 +740,28 @@ zval *php_gdk_bitmap_new(GdkWindow *bitmap)
 
 static void gdk_window_get_property(zval *return_value, zval *object, zend_llist_element **element, int *result)
 {
-	zval *value;
-	GdkWindow *win = PHP_GDK_WINDOW_GET(object);
+	GdkWindow *win;
 	gint x, y;
 	GdkModifierType p_mask;
 	char *prop_name = Z_STRVAL(((zend_overloaded_element *)(*element)->data)->element);
 	
 	*result = SUCCESS;
 
+	if (Z_OBJCE_P(object) == gdk_bitmap_ce) {
+		win = PHP_GDK_BITMAP_GET(object);
+	} else {
+		win = PHP_GDK_WINDOW_GET(object);
+	}
+
 	if (!strcmp(prop_name, "width")) {
 		gdk_window_get_size(win, &x, NULL);
 		RETURN_LONG(x);
 	} else if (!strcmp(prop_name, "height")) {
 		gdk_window_get_size(win, NULL, &y);
 		RETURN_LONG(y);
-	} else if (!strcmp(prop_name, "x")) {
-		gdk_window_get_position(win, &x, NULL);
-		RETURN_LONG(x);
-	} else if (!strcmp(prop_name, "y")) {
-		gdk_window_get_position(win, NULL, &y);
-		RETURN_LONG(y);
 	} else if (!strcmp(prop_name, "colormap")) {
 		*return_value = *php_gdk_colormap_new(gdk_window_get_colormap(win));
 		return;
-	} else if (!strcmp(prop_name, "pointer")) {
-		gdk_window_get_pointer(win, &x, &y, NULL);
-		*return_value = *php_gtk_build_value("(ii)", x, y);
-		return;
-	} else if (!strcmp(prop_name, "pointer_state")) {
-		gdk_window_get_pointer(win, NULL, NULL, &p_mask);
-		RETURN_LONG(p_mask);
-	} else if (!strcmp(prop_name, "parent")) {
-		GdkWindow *parent = gdk_window_get_parent(win);
-		if (parent)
-			*return_value = *php_gdk_window_new(parent);
-		return;
-	} else if (!strcmp(prop_name, "toplevel")) {
-		*return_value = *php_gdk_window_new(gdk_window_get_toplevel(win));
-		return;
-	} else if (!strcmp(prop_name, "children")) {
-		GList *children, *tmp;
-		zval *child;
-		children = gdk_window_get_children(win);
-		array_init(return_value);
-		for (tmp = children; tmp; tmp = tmp->next) {
-			child = php_gdk_window_new(tmp->data);
-			zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &child, sizeof(zval *), NULL);
-		}
-		g_list_free(children);
-		return;
 	} else if (!strcmp(prop_name, "type")) {
 		RETURN_LONG(gdk_window_get_type(win));
 	} else if (!strcmp(prop_name, "depth")) {
@@ -757,6 +774,42 @@ static void gdk_window_get_property(zval *return_value, zval *object, zend_llist
 	}
 #endif
 
+	if (Z_OBJCE_P(object) == gdk_window_ce) {
+		if (!strcmp(prop_name, "x")) {
+			gdk_window_get_position(win, &x, NULL);
+			RETURN_LONG(x);
+		} else if (!strcmp(prop_name, "y")) {
+			gdk_window_get_position(win, NULL, &y);
+			RETURN_LONG(y);
+		} else if (!strcmp(prop_name, "pointer")) {
+			gdk_window_get_pointer(win, &x, &y, NULL);
+			*return_value = *php_gtk_build_value("(ii)", x, y);
+			return;
+		} else if (!strcmp(prop_name, "pointer_state")) {
+			gdk_window_get_pointer(win, NULL, NULL, &p_mask);
+			RETURN_LONG(p_mask);
+		} else if (!strcmp(prop_name, "parent")) {
+			GdkWindow *parent = gdk_window_get_parent(win);
+			if (parent)
+				*return_value = *php_gdk_window_new(parent);
+			return;
+		} else if (!strcmp(prop_name, "toplevel")) {
+			*return_value = *php_gdk_window_new(gdk_window_get_toplevel(win));
+			return;
+		} else if (!strcmp(prop_name, "children")) {
+			GList *children, *tmp;
+			zval *child;
+			children = gdk_window_get_children(win);
+			array_init(return_value);
+			for (tmp = children; tmp; tmp = tmp->next) {
+				child = php_gdk_window_new(tmp->data);
+				zend_hash_next_index_insert(Z_ARRVAL_P(return_value), &child, sizeof(zval *), NULL);
+			}
+			g_list_free(children);
+			return;
+		}
+	}
+
 	*result = FAILURE;
 }
 
@@ -919,7 +972,7 @@ PHP_FUNCTION(gdk_colormap_alloc)
 	}
 
 	if (!gdk_colormap_alloc_color(PHP_GDK_COLORMAP_GET(this_ptr), &color, writeable, best_match)) {
-		php_error(E_WARNING, "%s() could not allocate color");
+		php_error(E_NOTICE, "%s() could not allocate color");
 		return;
 	}
 
@@ -1390,7 +1443,7 @@ static void release_gdk_gc_rsrc(zend_rsrc_list_entry *rsrc TSRMLS_DC)
 	gdk_gc_unref(obj);
 }
 
-
+#if 0
 static void gdk_colormap_query_color (GdkColormap *colormap, gulong pixel, GdkColor *result)
 {
 #ifndef PHP_WIN32
@@ -1436,6 +1489,7 @@ static void gdk_colormap_query_color (GdkColormap *colormap, gulong pixel, GdkCo
 			break;
 	}
 }
+#endif
 
 static void gdk_gc_get_property(zval *return_value, zval *object, zend_llist_element **element, int *result)
 {
@@ -1561,15 +1615,22 @@ static int gdk_gc_set_property(zval *object, zend_llist_element **element, zval
 			gdk_gc_set_font(gc, PHP_GDK_FONT_GET(value));
 		else
 			type_mismatch = 1;
-	} else if (php_gtk_check_class(value, gdk_window_ce) || Z_TYPE_P(value) == IS_NULL) {
-		GdkWindow *w = (Z_TYPE_P(value) == IS_NULL) ? NULL : PHP_GDK_WINDOW_GET(value);
-		
+	} else if (php_gtk_check_class(value, gdk_pixmap_ce) || php_gtk_check_class(value, gdk_bitmap_ce) || Z_TYPE_P(value) == IS_NULL) {
+		GdkDrawable *d = NULL;
+
+		if (Z_TYPE_P(value) != IS_NULL) {
+			if (php_gtk_check_class(value, gdk_pixmap_ce))
+				d = PHP_GDK_PIXMAP_GET(value);
+			else
+				d = PHP_GDK_BITMAP_GET(value);
+		}
+
 		if (!strcmp(prop_name, "tile"))
-			gdk_gc_set_tile(gc, w);
+			gdk_gc_set_tile(gc, d);
 		else if (!strcmp(prop_name, "stipple"))
-			gdk_gc_set_stipple(gc, w);
+			gdk_gc_set_stipple(gc, d);
 		else if (!strcmp(prop_name, "clip_mask"))
-			gdk_gc_set_clip_mask(gc, w);
+			gdk_gc_set_clip_mask(gc, d);
 		else
 			type_mismatch = 1;
 	} else
@@ -1684,8 +1745,8 @@ PHP_FUNCTION(gtk_selection_data_set)
 }
 
 static function_entry php_gtk_selection_data_functions[] = {
-	{"GdkSelectionData", PHP_FN(no_direct_constructor), NULL},
-	{"gdkselectiondata", PHP_FN(no_direct_constructor), NULL},
+	{"GtkSelectionData", PHP_FN(no_direct_constructor), NULL},
+	{"gtkselectiondata", PHP_FN(no_direct_constructor), NULL},
 	{"set", 			 PHP_FN(gtk_selection_data_set), NULL},
 	{NULL, NULL, NULL}
 };
@@ -1957,7 +2018,6 @@ PHP_FUNCTION(gtkstyle)
 PHP_FUNCTION(gtk_style_copy)
 {
 	GtkStyle *style;
-	zval *ret;
 
 	NOT_STATIC_METHOD();
 
@@ -2221,7 +2281,7 @@ static int style_helper_set(style_array_type type, gpointer array, zval *value,
 				case STYLE_PIXMAP_ARRAY:
 					{
 						GdkWindow **pixmap_array = (GdkWindow **)array;
-						if (!php_gtk_check_class(value, gdk_window_ce) && Z_TYPE_P(value) != IS_NULL) {
+						if (!php_gtk_check_class(value, gdk_pixmap_ce) && Z_TYPE_P(value) != IS_NULL) {
 							php_error(E_WARNING, "can only assign a GdkPixmap or null");
 							return PG_ERROR;
 						}
@@ -2343,7 +2403,6 @@ static int gtk_style_set_property(zval *object, zend_llist_element **element, zv
 }
 
 
-/* GtkBoxChild */
 static function_entry php_gtk_box_child_functions[] = {
 	{"GtkBoxChild", PHP_FN(no_direct_constructor), NULL},
 	{"gtkboxchild", PHP_FN(no_direct_constructor), NULL},
@@ -2609,7 +2668,7 @@ void php_gtk_plus_register_types(int module_number)
 	gdk_window_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
 	INIT_OVERLOADED_CLASS_ENTRY(ce, "GdkPixmap", php_gdk_pixmap_functions, NULL, php_gtk_get_property, php_gtk_set_property);
 	gdk_pixmap_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
-	INIT_OVERLOADED_CLASS_ENTRY(ce, "GdkBitmap", php_gdk_window_functions, NULL, php_gtk_get_property, php_gtk_set_property);
+	INIT_OVERLOADED_CLASS_ENTRY(ce, "GdkBitmap", php_gdk_bitmap_functions, NULL, php_gtk_get_property, php_gtk_set_property);
 	gdk_bitmap_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
 	php_gtk_register_prop_getter(gdk_window_ce, gdk_window_get_property);
 	php_gtk_register_prop_getter(gdk_pixmap_ce, gdk_window_get_property);
@@ -2664,19 +2723,19 @@ void php_gtk_plus_register_types(int module_number)
 	php_gtk_register_prop_getter(gtk_style_ce, gtk_style_get_property);
 	php_gtk_register_prop_setter(gtk_style_ce, gtk_style_set_property);
 
-	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkBoxChild", NULL, NULL, php_gtk_get_property, php_gtk_set_property);
+	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkBoxChild", php_gtk_box_child_functions, NULL, php_gtk_get_property, php_gtk_set_property);
 	gtk_box_child_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
 	php_gtk_register_prop_getter(gtk_box_child_ce, gtk_box_child_get_property);
 
-	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkTableChild", NULL, NULL, php_gtk_get_property, php_gtk_set_property);
+	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkTableChild", php_gtk_table_child_functions, NULL, php_gtk_get_property, php_gtk_set_property);
 	gtk_table_child_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
 	php_gtk_register_prop_getter(gtk_table_child_ce, gtk_table_child_get_property);
 
-	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkFixedChild", NULL, NULL, php_gtk_get_property, php_gtk_set_property);
+	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkFixedChild", php_gtk_fixed_child_functions, NULL, php_gtk_get_property, php_gtk_set_property);
 	gtk_fixed_child_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
 	php_gtk_register_prop_getter(gtk_fixed_child_ce, gtk_fixed_child_get_property);
 
-	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkCListRow", NULL, NULL, php_gtk_get_property, php_gtk_set_property);
+	INIT_OVERLOADED_CLASS_ENTRY(ce, "GtkCListRow", php_gtk_clist_row_functions, NULL, php_gtk_get_property, php_gtk_set_property);
 	gtk_clist_row_ce = zend_register_internal_class_ex(&ce, NULL, NULL TSRMLS_CC);
 	php_gtk_register_prop_getter(gtk_clist_row_ce, gtk_clist_row_get_property);
 }
