@@ -1,7 +1,7 @@
 /*
  * PHP-GTK - The PHP language bindings for GTK+
  *
- * Copyright (C) 2001 Andrei Zmievski <andrei@php.net>
+ * Copyright (C) 2001-2004 Andrei Zmievski <andrei@php.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
  
-/* $Id: php_scintilla.h,v 1.2 2001/12/08 02:35:47 mfischer Exp $: */
+/* $Id: php_scintilla.h,v 1.4.2.1 2004/08/07 06:49:35 andrei Exp $: */
 
 #ifndef PHP_SCINTILLA_H
 #define PHP_SCINTILLA_H
@@ -32,6 +32,9 @@
 extern php_gtk_ext_entry scintilla_ext_entry;
 #define scintilla_ext_ptr &scintilla_ext_entry
 
+void php_scintilla_register_constants(int module_number TSRMLS_DC);
+void php_scintilla_register_classes(void);
+
 #else
 
 #define scintilla_ext_ptr NULL
