@@ -2,7 +2,7 @@
 /*
  * PHP-GTK - The PHP language bindings for GTK+
  *
- * Copyright (C) 2001 Andrei Zmievski <andrei@php.net>
+ * Copyright (C) 2001-2004 Andrei Zmievski <andrei@php.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: arg_types.php,v 1.38 2002/01/04 17:45:35 fmk Exp $ */
+/* $Id: arg_types.php,v 1.41.2.1 2004/08/07 06:49:35 andrei Exp $ */
 
 /*======================================================================*\
 	Function: convert_typename
@@ -92,7 +92,7 @@ class Arg_Type {
 		trigger_error("This is an abstract class", E_USER_ERROR);
 	}
 
-	function write_from_prop($name)
+	function write_from_prop($name, $type)
 	{
 		trigger_error("This is an abstract class", E_USER_ERROR);
 	}
@@ -227,10 +227,10 @@ class Int_Arg extends Arg_Type {
 		return "	add_property_long($obj, \"$name\", $source);\n";
 	}
 
-	function write_from_prop($name)
+	function write_from_prop($name, $type)
 	{
 		return "	if (zend_hash_find(Z_OBJPROP_P(wrapper), \"$name\", sizeof(\"$name\"), (void **)&item) == SUCCESS && Z_TYPE_PP(item) == IS_LONG)\n" .
-        	   "		obj->$name = Z_LVAL_PP(item);\n" .
+        	   "		obj->$name = ($type)Z_LVAL_PP(item);\n" .
     		   "	else\n" .
         	   "		return 0;\n\n";
 
@@ -262,10 +262,10 @@ class Bool_Arg extends Arg_Type {
 		return "	add_property_bool($obj, \"$name\", $source);\n";
 	}
 
-	function write_from_prop($name)
+	function write_from_prop($name, $type)
 	{
 		return "	if (zend_hash_find(Z_OBJPROP_P(wrapper), \"$name\", sizeof(\"$name\"), (void **)&item) == SUCCESS && Z_TYPE_PP(item) == IS_BOOL)\n" .
-        	   "		obj->$name = Z_BVAL_PP(item);\n" .
+        	   "		obj->$name = ($type)Z_BVAL_PP(item);\n" .
     		   "	else\n" .
         	   "		return 0;\n\n";
 
@@ -281,7 +281,7 @@ class Double_Arg extends Arg_Type {
 		else
 			$var_list->add('double', $name);
 		$parse_list[] 	= '&' . $name;
-		$arg_list[] 	= $name;
+		$arg_list[] 	= "(float)$name";
 		return 'd';
 	}
 	
@@ -297,10 +297,10 @@ class Double_Arg extends Arg_Type {
 		return "	add_property_double($obj, \"$name\", $source);\n";
 	}
 
-	function write_from_prop($name)
+	function write_from_prop($name, $type)
 	{
 		return "	if (zend_hash_find(Z_OBJPROP_P(wrapper), \"$name\", sizeof(\"$name\"), (void **)&item) == SUCCESS && Z_TYPE_PP(item) == IS_DOUBLE)\n" .
-        	   "		obj->$name = Z_DVAL_PP(item);\n" .
+        	   "		obj->$name = ($type)Z_DVAL_PP(item);\n" .
     		   "	else\n" .
         	   "		return 0;\n\n";
 
@@ -606,8 +606,12 @@ class Drawable_Arg extends Arg_Type {
 				$extra_code[]	=	"	if (php_$name) {\n" .
 									"		if (Z_TYPE_P(php_$name) == IS_NULL)\n" .
 									"			$name = NULL;\n" .
-									"		else\n" .
-									"			$name = PHP_" . strtoupper($this->php_type). "_GET(php_" . $name . ");\n" .
+									"		else if (php_gtk_check_class(php_$name, gdk_window_ce))\n" .
+									"			$name = (GdkDrawable *)PHP_GDK_WINDOW_GET(php_$name);\n" .
+									"		else if(php_gtk_check_class(php_$name, gdk_pixmap_ce))\n" .
+									"			$name = (GdkDrawable *)PHP_GDK_PIXMAP_GET(php_$name);\n" .
+									"		else if(php_gtk_check_class(php_$name, gdk_bitmap_ce))\n" .
+									"			$name = (GdkDrawable *)PHP_GDK_BITMAP_GET(php_$name);\n" .
 									"	}\n";
 			} else {
 				$var_list->add($this->type, '*' . $name);
