@@ -2,7 +2,7 @@
 /*
  * PHP-GTK - The PHP language bindings for GTK+
  *
- * Copyright (C) 2001 Andrei Zmievski <andrei@php.net>
+ * Copyright (C) 2001-2004 Andrei Zmievski <andrei@php.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: generator.php,v 1.38 2002/01/23 02:22:48 andrei Exp $ */
+/* $Id: generator.php,v 1.43.2.1 2004/08/07 06:49:35 andrei Exp $ */
 
 /*
  * Significant portions of this generator are based on the pygtk code generator
@@ -29,6 +29,10 @@
 
 set_time_limit(300);
 
+// override the default PHP 8Mb as this script tends to use alot more
+// and hopefully reduce the support questions a bit..
+ini_set('memory_limit','64M');
+
 require "Getopt.php";
 require "arg_types.php";
 require "override.php";
@@ -308,8 +312,13 @@ class Generator {
 							   $function_call,
 							   $extra_post_code);
 
+		// hack to avoid macro conflict on win32
+		$extra_name = "";
+		if (strtolower($function->c_name) == "gdk_draw_pixmap")
+			$extra_name = "_int";
+
 		$function_code = sprintf($function_tpl,
-								 strtolower($function->c_name),
+								 strtolower($function->c_name) . $extra_name,
 								 $var_list->to_string(),
 								 $specs,
 								 $parse_list,
@@ -411,7 +420,7 @@ class Generator {
 				}
 
 				$init_prop_code[] = $handler->write_to_prop('result', $field_name, "obj->$field_name");
-				$get_prop_code[] = $handler->write_from_prop($field_name);
+				$get_prop_code[] = $handler->write_from_prop($field_name, $field_type);
 				$construct_prop_code[] = $handler->write_to_prop('this_ptr', $field_name, $field_name);
 
 				$specs .= $handler->write_param($field_type,
@@ -631,9 +640,15 @@ class Generator {
 						$function_name = substr($function->name, strlen($this->lprefix) + 1);
 					else
 						$function_name = $function->name;
+
+					// hack to avoid macro conflict on win32
+					$extra_name = "";
+					if (strtolower($function->c_name) == "gdk_draw_pixmap")
+						$extra_name = "_int";
+
 					$functions_decl .= sprintf($function_entry_tpl, 
 											   $function_name, 
-											   strtolower($function->c_name), 
+											   strtolower($function->c_name) . $extra_name, 
 											   'NULL');
 					$num_functions++;
 				}
@@ -763,6 +778,10 @@ foreach ($opts as $opt) {
 	}
 }
 
+if (file_exists(dirname($argv[1]) . '/arg_types.php')) {
+    include(dirname($argv[1]) . '/arg_types.php');
+}
+
 $parser = new Defs_Parser($argv[1]);
 $generator = new Generator($parser, $overrides, $prefix, $function_class);
 foreach ($register_defs as $defs) {
