@@ -1,7 +1,7 @@
 /*
  * PHP-GTK - The PHP language bindings for GTK+
  *
- * Copyright (C) 2001 Andrei Zmievski <andrei@php.net>
+ * Copyright (C) 2001-2004 Andrei Zmievski <andrei@php.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: php_gtk_object.c,v 1.48 2002/01/12 00:11:42 fmk Exp $ */
+/* $Id: php_gtk_object.c,v 1.56.2.2 2004/08/07 06:49:36 andrei Exp $ */
 
 #include "php_gtk.h"
 
@@ -44,7 +44,6 @@ PHP_GTK_API void php_gtk_object_init(GtkObject *obj, zval *wrapper)
 void php_gtk_set_object(zval *wrapper, void *obj, int rsrc_type)
 {
 	zval *handle;
-	zval **rsrc;
 
 	MAKE_STD_ZVAL(handle);
 	Z_TYPE_P(handle) = IS_LONG;
@@ -92,7 +91,7 @@ int php_gtk_get_simple_enum_value(zval *enum_val, int *result)
 	return 0;
 }
 
-int php_gtk_get_enum_value(GtkType enum_type, zval *enum_val, int *result)
+PHP_GTK_API int php_gtk_get_enum_value(GtkType enum_type, zval *enum_val, int *result)
 {
 	if (!enum_val)
 		return 0;
@@ -168,7 +167,6 @@ PHP_GTK_API void php_gtk_callback_marshal(GtkObject *o, gpointer data, guint nar
 	zval *wrapper = NULL;
 	zval *params;
 	zval *retval = NULL;
-	zval *tmp;
 	zval ***signal_args;
 	char *callback_name;
 	TSRMLS_FETCH();
@@ -217,7 +215,7 @@ PHP_GTK_API void php_gtk_callback_marshal(GtkObject *o, gpointer data, guint nar
 		MAKE_STD_ZVAL(params);
 		array_init(params);
 		zend_hash_next_index_insert(Z_ARRVAL_P(params), &wrapper, sizeof(zval *), NULL);
-		php_array_merge(Z_ARRVAL_P(params), Z_ARRVAL_P(gtk_args), 0);
+		php_array_merge(Z_ARRVAL_P(params), Z_ARRVAL_P(gtk_args), 0 TSRMLS_CC);
 		zval_ptr_dtor(&gtk_args);
 	} else
 		/* Otherwise, the only parameters will be GTK signal arguments. */
@@ -228,7 +226,7 @@ PHP_GTK_API void php_gtk_callback_marshal(GtkObject *o, gpointer data, guint nar
 	 * array.
 	 */
 	if (extra)
-		php_array_merge(Z_ARRVAL_P(params), Z_ARRVAL_PP(extra), 0);
+		php_array_merge(Z_ARRVAL_P(params), Z_ARRVAL_PP(extra), 0 TSRMLS_CC);
 	
 	signal_args = php_gtk_hash_as_array(params);
 
@@ -285,39 +283,6 @@ void php_gtk_handler_marshal(gpointer a, gpointer data, int nargs, GtkArg *args)
 		efree(handler_args);
 }
 
-void php_gtk_input_marshal(gpointer a, gpointer data, int nargs, GtkArg *args)
-{
-	zval *callback_data = (zval *)data;
-	zval *params;
-	zval *retval = NULL, **callback, **extra = NULL;
-	zval **callback_filename = NULL, **callback_lineno = NULL;
-	zval ***input_args;
-	char *callback_name;
-	TSRMLS_FETCH();
-
-	zend_hash_index_find(Z_ARRVAL_P(callback_data), 0, (void **)&callback);
-	zend_hash_index_find(Z_ARRVAL_P(callback_data), 1, (void **)&extra);
-	zend_hash_index_find(Z_ARRVAL_P(callback_data), 2, (void **)&callback_filename);
-	zend_hash_index_find(Z_ARRVAL_P(callback_data), 3, (void **)&callback_lineno);
-
-	if (!php_gtk_is_callable(*callback, 0, &callback_name)) {
-		php_error(E_WARNING, "Unable to call input callback '%s' specified in %s on line %d", callback_name, Z_STRVAL_PP(callback_filename), Z_LVAL_PP(callback_lineno));
-		efree(callback_name);
-		return;
-	}
-
-	params = php_gtk_args_as_hash(nargs, args);
-	if (extra)
-		php_array_merge(Z_ARRVAL_P(params), Z_ARRVAL_PP(extra), 0);
-	input_args = php_gtk_hash_as_array(params);
-
-	call_user_function_ex(EG(function_table), NULL, *callback, &retval, zend_hash_num_elements(Z_ARRVAL_P(params)), input_args, 0, NULL TSRMLS_CC);
-	if (retval)
-		zval_ptr_dtor(&retval);
-	efree(input_args);
-	zval_ptr_dtor(&params);
-}
-
 PHP_GTK_API void php_gtk_destroy_notify(gpointer user_data)
 {
 	zval *value = (zval *)user_data;
@@ -329,7 +294,6 @@ PHP_GTK_API zval *php_gtk_new(GtkObject *obj)
 	zval *wrapper;
 	zend_class_entry *ce;
 	GtkType type;
-	gchar *type_name;
 	TSRMLS_FETCH();
 	
 	if (!obj) {
@@ -440,7 +404,7 @@ int php_gtk_args_from_hash(GtkArg *args, int nparams, zval *hash)
 						   gtk_type_name(args[i].type),
 						   php_gtk_zval_type_name(*item));
 			else
-				g_snprintf(buf, 511, "argument %s: expected %s, %s found", i+1,
+				g_snprintf(buf, 511, "argument %d: expected %s, %s found", i+1,
 						   gtk_type_name(args[i].type),
 						   php_gtk_zval_type_name(*item));
 			php_error(E_WARNING, buf);
@@ -524,8 +488,8 @@ zval *php_gtk_arg_as_value(GtkArg *arg)
 			break;
 
 		case GTK_TYPE_POINTER:
-			php_error(E_NOTICE, "%s(): internal error: GTK_TYPE_POINTER unsupported",
-					  get_active_function_name(TSRMLS_C));
+			php_error(E_NOTICE, "%s(): internal error: %s GTK_TYPE_POINTER unsupported",
+					  get_active_function_name(TSRMLS_C), arg->name);
 			MAKE_STD_ZVAL(value);
 			ZVAL_NULL(value);
 			break;
@@ -631,7 +595,7 @@ int php_gtk_arg_from_value(GtkArg *arg, zval *value)
 
 		case GTK_TYPE_FLOAT:
 			convert_to_double(value);
-			GTK_VALUE_FLOAT(*arg) = Z_DVAL_P(value);
+			GTK_VALUE_FLOAT(*arg) = (gfloat)Z_DVAL_P(value);
 			break;
 
 		case GTK_TYPE_DOUBLE:
@@ -737,6 +701,7 @@ int php_gtk_arg_from_value(GtkArg *arg, zval *value)
 			break;
 
 		case GTK_TYPE_ARGS:
+		case GTK_TYPE_POINTER:
 		case GTK_TYPE_C_CALLBACK:
 			php_error(E_WARNING, "Unsupported type");
 			g_assert_not_reached();
@@ -907,7 +872,7 @@ void php_gtk_ret_from_value(GtkArg *ret, zval *value)
 
 		case GTK_TYPE_FLOAT:
 			convert_to_double(value);
-			*GTK_RETLOC_FLOAT(*ret) = Z_DVAL_P(value);
+			*GTK_RETLOC_FLOAT(*ret) = (gfloat)Z_DVAL_P(value);
 			break;
 
 		case GTK_TYPE_DOUBLE:
@@ -1032,7 +997,7 @@ static inline int invoke_setter(zval *object, zval *value, zend_llist_element **
 	return result;
 }
 
-zval php_gtk_get_property(zend_property_reference *property_reference)
+PHP_GTK_API zval php_gtk_get_property(zend_property_reference *property_reference)
 {
 	zval result;
 	zval *result_ptr = &result;
@@ -1049,7 +1014,8 @@ zval php_gtk_get_property(zend_property_reference *property_reference)
 		ZVAL_NULL(&result);
 		if (Z_TYPE_P(overloaded_property) == OE_IS_OBJECT) {
 			/* Trying to access a property on a non-object. */
-			if (Z_TYPE(object) != IS_OBJECT) {
+			if (Z_TYPE(object) != IS_OBJECT ||
+				Z_TYPE(overloaded_property->element) != IS_STRING) {
 				return result;
 			}
 
@@ -1257,5 +1223,17 @@ void php_gtk_call_function(INTERNAL_FUNCTION_PARAMETERS, zend_property_reference
 	zval_dtor(&method_name);
 }
 
+PHP_GTK_API void php_gtk_register_prop_getter(zend_class_entry *ce, prop_getter_t getter)
+{
+	zend_hash_index_update(&php_gtk_prop_getters, (long)ce, (void*)&getter,
+						   sizeof(prop_getter_t), NULL);
+}
+
+PHP_GTK_API void php_gtk_register_prop_setter(zend_class_entry *ce, prop_setter_t setter)
+{
+	zend_hash_index_update(&php_gtk_prop_setters, (long)ce, (void*)&setter,
+						   sizeof(prop_setter_t), NULL);
+}
+
 
 #endif  /* HAVE_PHP_GTK */
