@@ -1,7 +1,7 @@
 /*
  * PHP-GTK - The PHP language bindings for GTK+
  *
- * Copyright (C) 2001 Andrei Zmievski <andrei@php.net>
+ * Copyright (C) 2001-2004 Andrei Zmievski <andrei@php.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* $Id: php_gtk_util.c,v 1.24 2002/01/12 00:11:42 fmk Exp $ */
+/* $Id: php_gtk_util.c,v 1.27.4.1 2004/08/07 06:49:36 andrei Exp $ */
 
 #include "php_gtk.h"
 
@@ -358,10 +358,8 @@ static char *parse_arg_impl(zval **arg, va_list *va, char **spec, char *buf)
 static int parse_arg(int arg_num, zval **arg, va_list *va, char **spec, int quiet)
 {
 	char *expected_type;
-	char *actual_type;
 	char buf[1024];
 	char errorbuf[1024];
-	char objtype[1024];
 	TSRMLS_FETCH();
 
 	expected_type = parse_arg_impl(arg, va, spec, errorbuf);
@@ -523,7 +521,7 @@ int php_gtk_parse_args_hash_quiet(zval *hash, char *format, ...)
 	return retval;
 }
 
-int php_gtk_check_class(zval *wrapper, zend_class_entry *expected_ce)
+PHP_GTK_API int php_gtk_check_class(zval *wrapper, zend_class_entry *expected_ce)
 {
 	zend_class_entry *ce;
 
