@@ -1,12 +1,8 @@
 <?php
+/* $Id: fileselection.php,v 1.5 2002/12/04 17:31:50 fmk Exp $ */
 
-/* $Id: fileselection.php,v 1.3 2001/03/20 21:07:31 andrei Exp $ */
-
-if (!class_exists('gtk')) {
-	if (strtoupper(substr(PHP_OS, 0,3) == 'WIN'))
-		dl('php_gtk.dll');
-	else
-		dl('php_gtk.so');
+if (!extension_loaded('gtk')) {
+	dl( 'php_gtk.' . PHP_SHLIB_SUFFIX);
 }
 
 function shutdown() {
