@@ -1,11 +1,8 @@
 <?php
-/* $Id: gtk.php,v 1.40 2001/11/17 19:05:29 mfischer Exp $ */
+/* $Id: gtk.php,v 1.42 2002/12/04 17:31:50 fmk Exp $ */
 
-if (!class_exists('gtk')) {
-	if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN')
-		dl('php_gtk.dll');
-	else
-		dl('php_gtk.so');
+if (!extension_loaded('gtk')) {
+	dl( 'php_gtk.' . PHP_SHLIB_SUFFIX);
 }
 
 $windows = array();
@@ -2658,7 +2655,7 @@ function create_main_window()
 	$window->show_all();
 }
 
-Gtk::rc_parse('testgtkrc');
+Gtk::rc_parse(dirname($argv[0]).'/testgtkrc');
 create_main_window();
 Gtk::main();
 
