@@ -1,7 +1,9 @@
 <?php
+/* $Id: piemenu.php,v 1.3 2002/12/04 17:31:50 fmk Exp $ */
 
-    if (!extension_loaded('gtk'))
-        dl( 'php_gtk.' . (strstr( PHP_OS, 'WIN') ? 'dll' : 'so'));
+	if (!extension_loaded('gtk')) {
+		dl( 'php_gtk.' . PHP_SHLIB_SUFFIX);
+	}
 
     $w = &new GtkWindow;
     $w->set_title('GtkPieMenu Example');
